@@ -1,264 +1,201 @@
-""" Features that augment images
-
-Augmentations are features that can resolve more than one image without
-calling `.resolve()` on the parent feature. Specifically, they create
-`updates_per_reload` images, while calling their parent feature
-`load_size` times.
-
-Classes
--------
-Augmentation
-    Base abstract augmentation class.
-PreLoad
-    Simple storage with no augmentation.
-FlipLR
-    Flips images left-right.
-FlipUD
-    Flips images up-down.
-FlipDiagonal
-    Flips images diagonally.
+""" Features that augment images.
 """
 
-from .features import Feature
-from .image import Image
-from . import utils
-from .types import ArrayLike, PropertyLike
+import warnings
+import random
+from typing import Callable
 
 import numpy as np
 import scipy.ndimage as ndimage
-from scipy.ndimage.interpolation import map_coordinates
 from scipy.ndimage.filters import gaussian_filter
+from scipy.ndimage.interpolation import map_coordinates
 
-from typing import Callable
-import warnings
+from . import utils
+from .features import Feature
+from .image import Image
+from .types import ArrayLike, PropertyLike
 
 
 class Augmentation(Feature):
     """Base abstract augmentation class.
 
-    Augmentations are features that can resolve more than one image without
-    calling `.resolve()` on the parent feature. Specifically, they create
-    `updates_per_reload` images, while calling their parent feature
-    `load_size` times. They achieve this by resolving `load_size` results
-    from the parent feature at once, and randomly drawing one of these
-    results as input to the method `.get()`. A new input is chosen
-    every time `.update()` is called. Once `.update()` has been called
-    `updated_per_reload` times, a new batch of `load_size` results are
-    resolved from the parent feature.
-
-    The method `.get()` of implementations of this class may accept the
-    property `number_of_updates` as an argument. This number represents
-    the number of times the `.update()` method has been called since the
-    last time the parent feature was resolved.
-
     Parameters
     ----------
-    feature : Feature, optional, deprecated
-        DEPRECATED. The parent feature. If not passed, it acts like any other feature.
-    load_size : int
-        Number of results to resolve from the parent feature.
-    updates_per_reload : int
-        Number of times `.update()` is called before resolving new results
-        from the parent feature.
-    update_properties : Callable or None
-        Function called on the output of the method `.get()`. Overrides
-        the default behaviour, allowing full control over how to update
-        the properties of the output to account for the augmentation.
+    time_consistend: boolean
+       Whether to augment all images in a sequence equally.
     """
 
-    __distributed__ = False
+    def __init__(self, time_consistent=False, **kwargs):
+        super().__init__(time_consistent=time_consistent, **kwargs)
 
-    def __init__(
-        self,
-        feature: Feature = None,
-        load_size: PropertyLike[int] = 1,
-        updates_per_reload: PropertyLike[int] = 1,
-        update_properties: Callable or None = None,
-        **kwargs
-    ):
+    def _process_and_get(self, image_list, time_consistent, **kwargs):
 
-        if load_size != 1:
-            warnings.warn(
-                "Using an augmentation with a load size other than one is no longer supported",
-                DeprecationWarning,
-            )
-
-        self.feature = feature
-
-        def get_number_of_updates(updates_per_reload=1):
-            # Updates the number of updates. The very first update is not counted.
-            if not hasattr(self.properties["number_of_updates"], "_current_value"):
-                return 0
-            return (
-                self.properties["number_of_updates"].current_value + 1
-            ) % updates_per_reload
-
-        def tally():
-            idx = 0
-            while True:
-                yield idx
-                idx += 1
-
-        if not update_properties:
-            update_properties = self.update_properties
-
-        super().__init__(
-            load_size=load_size,
-            update_tally=tally(),
-            updates_per_reload=updates_per_reload,
-            index=kwargs.pop("index", False)
-            or (lambda load_size: np.random.randint(load_size)),
-            number_of_updates=get_number_of_updates,
-            update_properties=lambda: update_properties,
-            **kwargs
-        )
-
-    def _process_and_get(self, *args, update_properties=None, **kwargs):
-
-        # Loads a result from storage
-        if self.feature and (
-            not hasattr(self, "cache")
-            or kwargs["update_tally"] - self.last_update >= kwargs["updates_per_reload"]
-        ):
-
-            if isinstance(self.feature, list):
-                self.cache = [feature.resolve() for feature in self.feature]
-            else:
-                self.cache = self.feature.resolve()
-            self.last_update = kwargs["update_tally"]
-
-        if not self.feature:
-            image_list_of_lists = args[0]
+        if not isinstance(image_list, list):
+            wrap_depth = 2
+            image_list_of_lists = [[image_list]]
+        elif len(image_list) == 0 or not isinstance(image_list[0], list):
+            wrap_depth = 1
+            image_list_of_lists = [image_list]
         else:
-            image_list_of_lists = self.cache
-
-        if not isinstance(image_list_of_lists, list):
-            image_list_of_lists = [image_list_of_lists]
+            wrap_depth = 0
+            image_list_of_lists = image_list
 
         new_list_of_lists = []
-        # Calls get
-
-        np.random.seed(kwargs["hash_key"][0])
 
         for image_list in image_list_of_lists:
-            if isinstance(self.feature, list):
-                # If multiple features, ensure consistent rng
-                np.random.seed(kwargs["hash_key"][0])
 
-            if isinstance(image_list, list):
-                new_list_of_lists.append(
-                    [
-                        [
-                            Image(
-                                self.get(Image(image), **kwargs)
-                            ).merge_properties_from(image)
-                            for image in image_list
-                        ]
-                    ]
-                )
-            else:
-                # DANGEROUS
-                # if not isinstance(image_list, Image):
-                image_list = Image(image_list)
+            if time_consistent:
+                self.seed()
 
-                output = self.get(image_list, **kwargs)
+            augmented_list = []
+            for image in image_list:
+                self.seed()
+                augmented_image = Image(self.get(image, **kwargs))
+                augmented_image.merge_properties_from(image)
+                self.update_properties(augmented_image, **kwargs)
+                augmented_list.append(augmented_image)
 
-                if not isinstance(output, Image):
-                    output = Image(output)
-                new_list_of_lists.append(output.merge_properties_from(image_list))
+            new_list_of_lists.append(augmented_list)
 
-        if update_properties:
-            if not isinstance(new_list_of_lists, list):
-                new_list_of_lists = [new_list_of_lists]
-            for image_list in new_list_of_lists:
-                if not isinstance(image_list, list):
-                    image_list = [image_list]
-                for image in image_list:
-                    image.properties = [dict(prop) for prop in image.properties]
-                    update_properties(image, **kwargs)
+        for _ in range(wrap_depth):
+            new_list_of_lists = new_list_of_lists[0]
 
         return new_list_of_lists
 
-    def _update(self, **kwargs):
-        super()._update(**kwargs)
-        if self.feature and not self.number_of_updates.current_value:
-            if isinstance(self.feature, Feature):
-                self.feature._update(**kwargs)
-            elif isinstance(self.feature, list):
-                [feature._update(**kwargs) for feature in self.feature]
-
-    def update_properties(*args, **kwargs):
+    def update_properties(self, *args, **kwargs):
         pass
 
 
-class PreLoad(Augmentation):
-    """Simple storage with no augmentation.
+class Reuse(Feature):
+    """Acts like cache.
+
+    `Reuse` stores the output of a feature and reuses it for subsequent calls, even if it is updated.
+    This is can be used after a time-consuming feature to augment the output of the feature without
+    recalculating it. For example::
+
+       pipeline = dt.Reuse(pipeline, uses=2) >> dt.FlipLR()
+
+    Here, the output of pipeline is used twice, augmented randomly by FlipLR.
 
     Parameters
     ----------
     feature : Feature
-        The parent feature.
-    load_size : int
-        Number of results to resolve from the parent feature.
-    updates_per_reload : int
-        Number of times `.update()` is called before resolving new results
-        from the parent feature.
-    index : int
-        The index of the image to grab from storage. By default a random image.
-    update_properties : Callable or None
-        Function called on the output of the method `.get()`. Overrides
-        the default behaviour, allowing full control over how to update
-        the properties of the output to account for the augmentation.
+       The feature to reuse.
+    uses : int
+       Number of each stored image uses before evaluating `feature`. Note that the actual total number of uses is `uses * storage`. Should be constant.
+    storage : int
+       Number of instances of the output of `feature` to cache. Should be constant.
 
     """
 
-    def get(self, image, **kwargs):
-        return image
+    __distributed__ = False
+
+    def __init__(self, feature, uses=2, storage=1, **kwargs):
+
+        super().__init__(uses=uses, storage=storage, **kwargs)
+
+        self.feature = self.add_feature(feature)
+        self.counter = 0
+        self.cache = []
+
+    def get(self, image, uses, storage, **kwargs):
+
+        self.cache = self.cache[-storage:]
+
+        output = None
+
+        if len(self.cache) < storage or self.counter % (uses * storage) == 0:
+            output = self.feature(image)
+            self.cache.append(output)
+        else:
+            output = random.choice(self.cache)
+
+        self.counter += 1
+
+        if not isinstance(output, list):
+            output = [output]
+
+        outputs = []
+        for image in output:
+            image_copy = Image(image)
+            # shallow copy properties before output
+            image_copy.properties = [prop.copy() for prop in image.properties]
+            outputs.append(image_copy)
+
+        return outputs
 
 
 class FlipLR(Augmentation):
     """Flips images left-right.
 
     Updates all properties called "position" to flip the second index.
+
+    Arguments
+    ---------
+    p : float
+       Probability of flipping the image
+
+    Extra arguments
+    ---------------
+    augment : bool
+       Whether to perform the augmentation. Leaving as default is sufficient most of the time.
     """
 
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, load_size=1, updates_per_reload=2, **kwargs)
+    def __init__(self, p=0.5, augment=None, **kwargs):
+        super().__init__(
+            p=p,
+            augment=(lambda p: np.random.rand() < p) if augment is None else augment,
+            **kwargs,
+        )
 
-    def get(self, image, number_of_updates, **kwargs):
-        if number_of_updates % 2:
-            image = np.fliplr(image)
+    def get(self, image, augment, **kwargs):
+        if augment:
+            image = image[:, ::-1]
         return image
 
-    def update_properties(self, image, number_of_updates, **kwargs):
-        if number_of_updates % 2:
+    def update_properties(self, image, augment, **kwargs):
+        if augment:
             for prop in image.properties:
                 if "position" in prop:
                     position = np.array(prop["position"])
-                    position[..., 1] = image.shape[1] - position[..., 1]
+                    position[..., 1] = image.shape[1] - position[..., 1] - 1
                     prop["position"] = position
 
 
 class FlipUD(Augmentation):
     """Flips images up-down.
 
-    Updates all properties called "position" by flipping the first index.
+    Updates all properties called "position" to flip the first index.
+
+    Arguments
+    ---------
+    p : float
+       Probability of flipping the image
+
+    Extra arguments
+    ---------------
+    augment : bool
+       Whether to perform the augmentation. Leaving as default is sufficient most of the time.
     """
 
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, load_size=1, updates_per_reload=2, **kwargs)
+    def __init__(self, p=0.5, augment=None, **kwargs):
+        super().__init__(
+            p=p,
+            augment=(lambda p: np.random.rand() < p) if augment is None else augment,
+            **kwargs,
+        )
 
-    def get(self, image, number_of_updates=0, **kwargs):
-        if number_of_updates % 2:
-            image = np.flipud(image)
+    def get(self, image, augment, **kwargs):
+        if augment:
+            image = image[::-1]
         return image
 
-    def update_properties(self, image, number_of_updates, **kwargs):
-        if number_of_updates % 2:
+    def update_properties(self, image, augment, **kwargs):
+        if augment:
             for prop in image.properties:
                 if "position" in prop:
                     position = np.array(prop["position"])
-                    position[..., 0] = image.shape[0] - position[..., 0]
+                    position[..., 0] = image.shape[0] - position[..., 0] - 1
                     prop["position"] = position
 
 
@@ -266,18 +203,32 @@ class FlipDiagonal(Augmentation):
     """Flips images along the main diagonal.
 
     Updates all properties called "position" by swapping the first and second index.
+
+    Arguments
+    ---------
+    p : float
+       Probability of flipping the image
+
+    Extra arguments
+    ---------------
+    augment : bool
+       Whether to perform the augmentation. Leaving as default is sufficient most of the time.
     """
 
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, load_size=1, updates_per_reload=2, **kwargs)
+    def __init__(self, p=0.5, augment=None, **kwargs):
+        super().__init__(
+            p=p,
+            augment=(lambda p: np.random.rand() < p) if augment is None else augment,
+            **kwargs,
+        )
 
-    def get(self, image, number_of_updates, axes=(1, 0, 2), **kwargs):
-        if number_of_updates % 2:
-            image = np.transpose(image, axes=axes)
+    def get(self, image, augment, **kwargs):
+        if augment:
+            image = np.transpose(image, axes=(1, 0, *range(2, image.ndim)))
         return image
 
-    def update_properties(self, image, number_of_updates, **kwargs):
-        if number_of_updates % 2:
+    def update_properties(self, image, augment, **kwargs):
+        if augment:
             for prop in image.properties:
                 if "position" in prop:
                     position = np.array(prop["position"])
@@ -358,7 +309,7 @@ class Affine(Augmentation):
             order=order,
             cval=cval,
             mode=mode,
-            **kwargs
+            **kwargs,
         )
 
     def _process_properties(self, properties):
@@ -423,7 +374,7 @@ class Affine(Augmentation):
                 input=image,
                 matrix=mapping,
                 offset=d,
-                **kwargs
+                **kwargs,
             )
 
             new_image = Image(new_image)
@@ -437,7 +388,7 @@ class Affine(Augmentation):
                     input=image[:, :, z],
                     matrix=mapping,
                     offset=d,
-                    **kwargs
+                    **kwargs,
                 )
 
         # Map positions
@@ -540,7 +491,7 @@ class ElasticTransformation(Augmentation):
             order=order,
             cval=cval,
             mode=mode,
-            **kwargs
+            **kwargs,
         )
 
     def get(self, image, sigma, alpha, ignore_last_dim, **kwargs):
@@ -579,7 +530,7 @@ class ElasticTransformation(Augmentation):
                     map_coordinates,
                     input=image[..., z],
                     coordinates=coordinates,
-                    **kwargs
+                    **kwargs,
                 ).reshape(shape)
         else:
             image = utils.safe_call(
@@ -708,7 +659,7 @@ class CropToMultiplesOf(Crop):
 
         def image_to_crop(image):
             shape = image.shape
-            multiple = self.multiple.current_value
+            multiple = self.multiple()
 
             if not isinstance(multiple, (list, tuple, np.ndarray)):
                 multiple = (multiple,) * image.ndim
@@ -726,7 +677,7 @@ class CropToMultiplesOf(Crop):
             corner=corner,
             crop=lambda: image_to_crop,
             crop_mode="retain",
-            **kwargs
+            **kwargs,
         )
 
 
@@ -775,15 +726,8 @@ class Pad(Augmentation):
         results = [self.get(image, **kwargs) for image in images]
         for idx, result in enumerate(results):
             if isinstance(result, tuple):
-                shape = result[0].shape
-                padding = result[1]
-                de_pad = tuple(
-                    slice(p[0], shape[dim] - p[1]) for dim, p in enumerate(padding)
-                )
-                results[idx] = (
-                    Image(result[0]).merge_properties_from(images[idx]),
-                    {"undo_padding": de_pad},
-                )
+
+                results[idx] = Image(result[0]).merge_properties_from(images[idx])
             else:
                 Image(results[idx]).merge_properties_from(images[idx])
         return results
@@ -804,7 +748,7 @@ class PadToMultiplesOf(Pad):
     def __init__(self, multiple: PropertyLike[int or None] = 1, **kwargs):
         def amount_to_pad(image):
             shape = image.shape
-            multiple = self.multiple.current_value
+            multiple = self.multiple()
 
             if not isinstance(multiple, (list, tuple, np.ndarray)):
                 multiple = (multiple,) * image.ndim
