@@ -20,7 +20,11 @@ Ellipsoid
 
 from pint import Quantity
 from . import image
-from deeptrack.backend.units import ConversionTable
+from deeptrack.backend.units import (
+    ConversionTable,
+    get_active_scale,
+    get_active_voxel_size,
+)
 from typing import Callable, Tuple
 
 import numpy as np
@@ -74,7 +78,9 @@ class Scatterer(Feature):
     __list_merge_strategy__ = MERGE_STRATEGY_APPEND
     __distributed__ = False
     __conversion_table__ = ConversionTable(
-        position=(u.pixel, u.pixel), z=(u.pixel, u.pixel), voxel_size=(u.meter, u.meter)
+        position=(u.pixel, u.pixel),
+        z=(u.zpixel, u.zpixel),
+        voxel_size=(u.meter, u.meter),
     )
 
     def __init__(
@@ -86,8 +92,13 @@ class Scatterer(Feature):
         upsample: PropertyLike[int] = 1,
         voxel_size=None,
         pixel_size=None,
-        **kwargs
+        **kwargs,
     ):
+        # Ignore warning to help with comparison with arrays.
+        if upsample is not 1:  # noqa: F632
+            warnings.warn(
+                f"Setting upsample != 1 is deprecated. Please, instead use dt.Upscale(f, factor={upsample})"
+            )
         self._processed_properties = False
         super().__init__(
             position=position,
@@ -119,17 +130,14 @@ class Scatterer(Feature):
                 + "Optics.upscale != 1."
             )
 
-        # Calculates upsampled voxel_size
-        if upsample_axes is None:
-            upsample_axes = range(3)
-
-        voxel_size = np.array(voxel_size)
-        for axis in upsample_axes:
-            voxel_size[axis] /= upsample
+        voxel_size = get_active_voxel_size()
 
         # calls parent _process_and_get
         new_image = super()._process_and_get(
-            *args, voxel_size=voxel_size, upsample=upsample, **kwargs
+            *args,
+            voxel_size=voxel_size,
+            upsample=upsample,
+            **kwargs,
         )
         new_image = new_image[0]
 
@@ -142,28 +150,28 @@ class Scatterer(Feature):
                 Warning,
             )
 
-        # Downsamples the image along the axes it was upsampled
-        if upsample != 1 and upsample_axes:
+        # # Downsamples the image along the axes it was upsampled
+        # if upsample != 1 and upsample_axes:
 
-            # Pad image to ensure it is divisible by upsample
-            increase = np.array(new_image.shape)
-            for axis in upsample_axes:
-                increase[axis] = upsample - (new_image.shape[axis] % upsample)
-            pad_width = [(0, inc) for inc in increase]
-            new_image = np.pad(new_image, pad_width, mode="constant")
+        #     # Pad image to ensure it is divisible by upsample
+        #     increase = np.array(new_image.shape)
+        #     for axis in upsample_axes:
+        #         increase[axis] = upsample - (new_image.shape[axis] % upsample)
+        #     pad_width = [(0, inc) for inc in increase]
+        #     new_image = np.pad(new_image, pad_width, mode="constant")
 
-            # Finds reshape size for downsampling
-            new_shape = []
-            for axis in range(new_image.ndim):
-                if axis in upsample_axes:
-                    new_shape += [new_image.shape[axis] // upsample, upsample]
-                else:
-                    new_shape += [new_image.shape[axis]]
+        #     # Finds reshape size for downsampling
+        #     new_shape = []
+        #     for axis in range(new_image.ndim):
+        #         if axis in upsample_axes:
+        #             new_shape += [new_image.shape[axis] // upsample, upsample]
+        #         else:
+        #             new_shape += [new_image.shape[axis]]
 
-            # Downsamples
-            new_image = np.reshape(new_image, new_shape).mean(
-                axis=tuple(np.array(upsample_axes, dtype=np.int32) * 2 + 1)
-            )
+        #     # Downsamples
+        #     new_image = np.reshape(new_image, new_shape).mean(
+        #         axis=tuple(np.array(upsample_axes, dtype=np.int32) * 2 + 1)
+        #     )
 
         # Crops empty slices
         if crop_empty:
@@ -196,12 +204,11 @@ class PointParticle(Scatterer):
     """
 
     def __init__(self, **kwargs):
-        kwargs.pop("upsample", False)
-        kwargs.pop("upsample_axes", False)
         super().__init__(upsample=1, upsample_axes=(), **kwargs)
 
     def get(self, image, **kwargs):
-        return np.ones((1, 1, 1))
+        scale = get_active_scale()
+        return np.ones((1, 1, 1)) * np.prod(scale)
 
 
 class Ellipse(Scatterer):
@@ -227,10 +234,14 @@ class Ellipse(Scatterer):
         for `Brightfield` and `intensity` for `Fluorescence`).
     upsample : int
         Upsamples the calculations of the pixel occupancy fraction.
+    transpose : bool
+        If True, the ellipse is transposed as to align the first axis of the radius with
+        the first axis of the created volume. This is applied before rotation.
+
     """
 
     __conversion_table__ = ConversionTable(
-        radius=(u.meter, u.pixel),
+        radius=(u.meter, u.meter),
         rotation=(u.radian, u.radian),
     )
 
@@ -238,10 +249,11 @@ class Ellipse(Scatterer):
         self,
         radius: PropertyLike[float] = 1e-6,
         rotation: PropertyLike[float] = 0,
-        **kwargs
+        transpose: PropertyLike[bool] = False,
+        **kwargs,
     ):
         super().__init__(
-            radius=radius, rotation=rotation, upsample_axes=(0, 1), **kwargs
+            radius=radius, rotation=rotation, transpose=transpose, **kwargs
         )
 
     def _process_properties(self, properties: dict) -> dict:
@@ -265,12 +277,18 @@ class Ellipse(Scatterer):
 
         return properties
 
-    def get(self, *ignore, radius, rotation, voxel_size, **kwargs):
+    def get(self, *ignore, radius, rotation, voxel_size, transpose, **kwargs):
 
+        if not transpose:
+            radius = radius[::-1]
+            # rotation = rotation[::-1]
         # Create a grid to calculate on
         rad = radius[:2]
-        ceil = int(np.max(np.ceil(rad)))
-        X, Y = np.meshgrid(np.arange(-ceil, ceil), np.arange(-ceil, ceil))
+        ceil = int(np.ceil(np.max(rad) / np.min(voxel_size[:2])))
+        Y, X = np.meshgrid(
+            np.arange(-ceil, ceil) * voxel_size[1],
+            np.arange(-ceil, ceil) * voxel_size[0],
+        )
 
         # Rotate the grid
         if rotation != 0:
@@ -308,7 +326,7 @@ class Sphere(Scatterer):
     """
 
     __conversion_table__ = ConversionTable(
-        radius=(u.meter, u.pixel),
+        radius=(u.meter, u.meter),
     )
 
     def __init__(self, radius: PropertyLike[float] = 1e-6, **kwargs):
@@ -317,15 +335,14 @@ class Sphere(Scatterer):
     def get(self, image, radius, voxel_size, **kwargs):
 
         # Create a grid to calculate on
-        rad = radius * np.ones(3)
+        rad = radius * np.ones(3) / voxel_size
         rad_ceil = np.ceil(rad)
         x = np.arange(-rad_ceil[0], rad_ceil[0])
         y = np.arange(-rad_ceil[1], rad_ceil[1])
         z = np.arange(-rad_ceil[2], rad_ceil[2])
-        X, Y, Z = np.meshgrid((x / rad[0]) ** 2, (y / rad[1]) ** 2, (z / rad[2]) ** 2)
+        X, Y, Z = np.meshgrid((y / rad[1]) ** 2, (x / rad[0]) ** 2, (z / rad[2]) ** 2)
 
         mask = (X + Y + Z <= 1) * 1.0
-
         return mask
 
 
@@ -352,10 +369,13 @@ class Ellipsoid(Scatterer):
         for `Brightfield` and `intensity` for `Fluorescence`).
     upsample : int
         Upsamples the calculations of the pixel occupancy fraction.
+    transpose : bool
+        If True, the ellipse is transposed as to align the first axis of the radius with
+        the first axis of the created volume. This is applied before rotation.
     """
 
     __conversion_table__ = ConversionTable(
-        radius=(u.meter, u.pixel),
+        radius=(u.meter, u.meter),
         rotation=(u.radian, u.radian),
     )
 
@@ -363,9 +383,12 @@ class Ellipsoid(Scatterer):
         self,
         radius: PropertyLike[float] = 1e-6,
         rotation: PropertyLike[float] = 0,
-        **kwargs
+        transpose: PropertyLike[bool] = False,
+        **kwargs,
     ):
-        super().__init__(radius=radius, rotation=rotation, **kwargs)
+        super().__init__(
+            radius=radius, rotation=rotation, transpose=transpose, **kwargs
+        )
 
     def _process_properties(self, propertydict):
         """Preprocess the input to the method .get()
@@ -414,18 +437,21 @@ class Ellipsoid(Scatterer):
 
         return propertydict
 
-    def get(self, image, radius, rotation, voxel_size, **kwargs):
+    def get(self, image, radius, rotation, voxel_size, transpose, **kwargs):
+        if not transpose:
+            # swap the first and second value of the radius vector
+            radius = (radius[1], radius[0], radius[2])
 
-        radius_in_pixels = radius
+        # radius_in_pixels = np.array(radius) / np.array(voxel_size)
 
-        max_rad = np.max(radius)
-        rad_ceil = np.ceil(max_rad)
+        # max_rad = np.max(radius_in_pixels)
+        rad_ceil = np.ceil(np.max(radius) / np.min(voxel_size))
 
         # Create grid to calculate on
-        x = np.arange(-rad_ceil, rad_ceil)
-        y = np.arange(-rad_ceil, rad_ceil)
-        z = np.arange(-rad_ceil, rad_ceil)
-        X, Y, Z = np.meshgrid(x, y, z)
+        x = np.arange(-rad_ceil, rad_ceil) * voxel_size[0]
+        y = np.arange(-rad_ceil, rad_ceil) * voxel_size[1]
+        z = np.arange(-rad_ceil, rad_ceil) * voxel_size[2]
+        Y, X, Z = np.meshgrid(y, x, z)
 
         # Rotate the grid
         cos = np.cos(rotation)
@@ -443,12 +469,8 @@ class Ellipsoid(Scatterer):
         ZR = (-sin[1] * X) + cos[1] * sin[2] * Y + cos[1] * cos[2] * Z
 
         mask = (
-            (XR / radius_in_pixels[0]) ** 2
-            + (YR / radius_in_pixels[1]) ** 2
-            + (ZR / radius_in_pixels[2]) ** 2
-            < 1
+            (XR / radius[0]) ** 2 + (YR / radius[1]) ** 2 + (ZR / radius[2]) ** 2 < 1
         ) * 1.0
-
         return mask
 
 
@@ -519,7 +541,7 @@ class MieScatterer(Scatterer):
         padding=(0,) * 4,
         output_region=None,
         polarization_angle=None,
-        **kwargs
+        **kwargs,
     ):
         if polarization_angle is not None:
             warnings.warn(
@@ -589,14 +611,16 @@ class MieScatterer(Scatterer):
         input_polarization,
         output_polarization,
         coefficients,
-        **kwargs
+        **kwargs,
     ):
 
         xSize = padding[2] + output_region[2] - output_region[0] + padding[0]
         ySize = padding[3] + output_region[3] - output_region[1] + padding[1]
 
+        scale = get_active_scale()
+
         arr = pad_image_to_fft(np.zeros((xSize, ySize)))
-        position = np.array(position)
+        position = np.array(position) * scale[: len(position)]
         pos_floor = np.floor(position)
         pos_digits = position - pos_floor
         # Evluation grid
@@ -717,7 +741,7 @@ class MieSphere(MieScatterer):
         self,
         radius: PropertyLike[float] = 1e-6,
         refractive_index: PropertyLike[float] = 1.45,
-        **kwargs
+        **kwargs,
     ):
         def coeffs(radius, refractive_index, refractive_index_medium, wavelength):
             def inner(L):
@@ -787,7 +811,7 @@ class MieStratifiedSphere(MieScatterer):
         self,
         radius: PropertyLike[ArrayLike[float]] = [1e-6],
         refractive_index: PropertyLike[ArrayLike[float]] = [1.45],
-        **kwargs
+        **kwargs,
     ):
         def coeffs(radius, refractive_index, refractive_index_medium, wavelength):
             assert np.all(
